#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `badgr_lite` package."""


import os
import json
import tempfile
import unittest

from click.testing import CliRunner
import vcr

from badgr_lite import cli


class TestBadgrLiteBase(unittest.TestCase):
    """Test Base for setUp tearDown and test helper methods"""

    SAMPLE_TOKEN_DATA = {
        "access_token": "FVQ__sample_token__QYzzRracgjH",
        "token_type": "Bearer",
        "expires_in": 86400,
        "refresh_token": "vK__sample_refresh_token__AlPZ",
        "scope": "rw:profile rw:issuer rw:backpack"}

    def setUp(self):
        self.runner = CliRunner()
        self.create_token_file()
        self.cli_options = [
            '--token-file', self.token_file,
            'award-badge',
            '--badge-id', '2TfNNqMLT8CoAhfGKqSv6Q',
            '--recipient', 'recipient@example.com',
            '--notify',
            '--evidence-url', 'https://example.com',
            '--evidence-narrative', 'John Doe performed...']

    def tearDown(self):
        os.remove(self.token_file)

    def create_token_file(self) -> None:
        """Create token file for tests

        Although this is slightly more like a factory than a fixture, it
        feels like a more crude way to do the tests. Instead of making a
        fake token file, it feels like the approach should be to mock
        out the calls that access the token data.

        However, as of the time of this writing, the initial author of
        this library was not able to cleanly do this. Since the great is
        often the enemy of the good, we will leave this functioning
        approach.

        There is a good opportunity to refactor these tests to make them
        more streamlined so that creating a temporary token file is not
        needed.
        """
        _, self.token_file = tempfile.mkstemp(suffix='.json', prefix='token')
        with open(self.token_file, 'w') as tmp_file:
            tmp_file.write(json.dumps(TestBadgrLiteBase.SAMPLE_TOKEN_DATA))


class TestBadgrLiteCLI(TestBadgrLiteBase):
    """Tests for `badgr_lite` package."""

    def test_cli_main_entry_point(self):
        """Installation calls into cli.main()"""

        result = self.runner.invoke(cli.main)
        self.assertEqual(0, result.exit_code)
        self.assertIn(
            'Automate Badgr tasks without the overhead of badgr-server',
            result.output)

    def test_cli_token_file_option(self):
        """CLI takes --token argument"""

        help_result = self.runner.invoke(cli.main, ['--help'])
        self.assertEqual(0, help_result.exit_code)
        self.assertIn(
            '--token-file PATH  File holding token credentials',
            help_result.output)

    def test_cli_help_shows_subcommand_without_token(self):
        """CLI shows subcommands help without token

        If the `--token-file` option were required with
        `type=click.Path(exists=True)`, then the --token-file argument
        would need to point to a valid token before one could ask for
        syntactial help:

        >>> badgr list-badges --help

        We should be able to get help for the subcommand before the
        token file existence is enforced.
        """

        result = self.runner.invoke(cli.main, ['list-badges', '--help'])
        self.assertEqual(0, result.exit_code)

    def test_cli_subcommand_with_token(self):
        """CLI subcommands should take --token-file argument

        Even though we should still get `--help` on subcommands, the
        subcommands should use the `--token-file` argument (or
        it's default).
        """

        _, token_file = tempfile.mkstemp(suffix='.json', prefix='token')
        result = self.runner.invoke(
            cli.main,
            ['--token-file', token_file, 'list-badges', '--help'])
        self.assertEqual(0, result.exit_code)
        os.remove(token_file)


class TestBadgrLiteCLIListBadges(TestBadgrLiteBase):
    """BadgrLite CLI list-badge subcommand tests

    See also .models badges property tests
    """

    def test_cli_subcommand_list_badges_help(self):
        """CLI has subcommand list-badges"""

        result = self.runner.invoke(cli.main, ['list-badges', '--help'])
        self.assertEqual(0, result.exit_code)


class TestBadgrLiteCLIAwardBadge(TestBadgrLiteBase):
    """BadgrLite CLI award-badge subcommand tests"""

    def test_cli_subcommand_award_badge_help(self):
        """CLI has subcommand award-badge"""

        result = self.runner.invoke(cli.main, ['award-badge', '--help'])
        self.assertEqual(0, result.exit_code)

    def test_cli_subcommand_award_badge_badge_id(self):
        """CLI award-badge requires --badge-id

        If a Badge ID is not provided, it will be prompted.

        In this test, we'll add "Some Badge ID" for the
        prompt input.
        """

        with vcr.use_cassette('tests/vcr_cassettes/award_badge.yaml'):
            self.cli_options.remove('--badge-id')
            self.cli_options.remove('2TfNNqMLT8CoAhfGKqSv6Q')
            result = self.runner.invoke(
                cli.main, self.cli_options,
                input="2TfNNqMLT8CoAhfGKqSv6Q")
            self.assertEqual(0, result.exit_code)
            self.assertIn('Badge ID: 2TfNNqMLT8CoAhfGKqSv6Q', result.output)

    def test_cli_subcommand_award_badge_recipient(self):
        """CLI award-badge requires --recipient

        If a recipient is not provided, it will be prompted.

        In this test, we'll add "recipient@example.com" for the
        prompt input.
        """

        with vcr.use_cassette('tests/vcr_cassettes/award_badge.yaml'):
            self.cli_options.remove('--recipient')
            self.cli_options.remove('recipient@example.com')
            result = self.runner.invoke(
                cli.main, self.cli_options,
                input="recipient@example.com")
            self.assertEqual(0, result.exit_code)
            self.assertIn('Recipient email: recipient@example.com',
                          result.output)

    def test_cli_subcommand_award_badge_notify(self):
        """CLI award-badge allows --notify / --no-notify"""

        with vcr.use_cassette('tests/vcr_cassettes/award_badge.yaml'):
            result = self.runner.invoke(cli.main, self.cli_options)
            self.assertEqual(0, result.exit_code)

    def test_cli_subcommand_award_badge_evidence_url(self):
        """CLI award-badge allows --evidence_url"""

        with vcr.use_cassette('tests/vcr_cassettes/award_badge.yaml'):
            result = self.runner.invoke(cli.main, self.cli_options)
            # See also self.cli_options.
            self.assertEqual(0, result.exit_code)

    def test_cli_subcommand_award_badge_evidence_narrative(self):
        """CLI award-badge allows --evidence-narrative"""

        with vcr.use_cassette('tests/vcr_cassettes/award_badge.yaml'):
            result = self.runner.invoke(cli.main, self.cli_options)
            # See also self.cli_options.
            self.assertEqual(0, result.exit_code)

    def test_cli_xor(self):
        """CLI has Exclusive OR (XOR) function

        This CLI needs one, or the other, but not both:

        A | B   | Result
        --+-+---+-------
        T | T   | False
        T | F   | True
        F | F   | True
        F | F   | False
        """

        self.assertEqual(cli.xor(True, True), False)
        self.assertEqual(cli.xor(True, False), True)
        self.assertEqual(cli.xor(False, True), True)
        self.assertEqual(cli.xor(False, False), False)

    def test_cli_subcommand_award_badge_xor_evidence(self):
        """CLI award-badge evidence requires URL and Narrative

        If either `--evidence-url` or `--evidence-narrative` is provided,
        require both.
        """
        with vcr.use_cassette('tests/vcr_cassettes/award_badge.yaml'):
            self.cli_options.remove('--evidence-narrative')
            self.cli_options.remove('John Doe performed...')
            result = self.runner.invoke(cli.main, self.cli_options)
            self.assertNotEqual(0, result.exit_code)
            self.assertIn(
                "If one evidence paramater is used, both are needed",
                result.output)

    def test_cli_subcommand_award_badge_ensure_evidence(self):
        """CLI has ensure_evidence function

        Function ensures there is an 'evidence' key and the associated value is
        a list.
        """
        badge_data = cli.ensure_evidence({})
        self.assertIn('evidence', badge_data.keys())
        self.assertTrue(isinstance(badge_data['evidence'], list))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""BadgrLite module for automating Badr awards (assertions)"""

# pylint: disable=R1710
# pylint: disable=R1710

import json
import os

import requests
from requests.models import Response

from badgr_lite import exceptions
from .helpers import pythonic, to_datetime


class Badge:
    """Pythonic representation of API BadgeClass

    Given a dictionary when instantiating the object, create a Pythonic
    representation of a OpenBadge.

    The JSON object given by the Badgr API, loaded as a dict, can be used to
    instantiate the Badge class.
    """
    # There are enough public methods; pylint: disable=R0903
    # Attrs are dynamically assigned;  pylint: disable=E1101

    REQUIRED_JSON = ['entityId', 'expires', 'entityType', 'extensions',
                     'openBadgeId', 'createdBy', 'issuer', 'image',
                     'issuerOpenBadgeId', 'createdAt']
    REQUIRED_ATTRS = [pythonic(attr) for attr in REQUIRED_JSON]

    def __init__(self, attrs: dict) -> None:
        """Initialize with single dictionary

        Pythonic attributes are created from the given attrs dictionary.

        Note that the keys in REQUIRED_ATTRS (or their JSON equivalents)
        are required for initialization. The RequiredAttributesMissin...
        exception is raised if they are not all present.

        Also, `created_at` (createdAt) is converted from string to
        datetime.
        """
        pythonic_attrs = {pythonic(k): v for k, v in attrs.items()}
        if "created_at" in pythonic_attrs:
            converted_value = to_datetime(pythonic_attrs['created_at'])
            pythonic_attrs['created_at'] = converted_value

        for pythonic_key, value in pythonic_attrs.items():
            setattr(self, pythonic_key, value)

        self._attrs = pythonic_attrs
        self._check_missing_but_required(pythonic_attrs)

    def _check_missing_but_required(self, pythonic_attrs: dict) -> None:
        """Raise exception if required attributes not in pythonic_attrs

        Raise exceptions.RequiredAttributesMissingError if required attributes
        are not given.
        """
        missing_but_required =\
            set(self.REQUIRED_ATTRS) - set(pythonic_attrs.keys())

        if missing_but_required:
            raise exceptions.RequiredAttributesMissingError(
                ", ".join(missing_but_required))

    def __str__(self):
        url = "https://badgr.io/public/assertions/{}".format(self.entity_id)
        name = "<No name>"
        if hasattr(self, 'name'):
            name = self.name

        return "{}\t{}\t{}".format(self.entity_id, url, name)


class BadgrLite:
    """Automate using Badgr API without the overhead of badgr-server"""
    # pylint: disable=R0903

    def __init__(self, token_filename: str) -> None:
        self.token_filename = token_filename
        self._token_data = None

    def load_token(self) -> None:
        """Given initialization with token_filename, load token data

        Ensure token_filename exists. Load JSON data from the filename.
        Store in self._token_data
        """
        if not os.path.exists(self.token_filename):
            raise exceptions.TokenFileNotFoundError(
                "Token File Not Found.",
                exceptions.TokenFileNotFoundError.__doc__)

        with open(self.token_filename, 'r') as token_handler:
            self._token_data = json.load(token_handler)

    def refresh_token(self):
        """Refresh access token from refresh_token"""

        response = requests.post(
            'https://api.badgr.io/o/token',
            data={'grant_type': 'refresh_token',
                  'refresh_token': self._token_data['refresh_token']})

        # An else after a raise is perfectly valid here; pylint: disable=R1720
        if response.status_code == 401:
            raise exceptions.TokenAndRefreshExpiredError
        else:
            assert response.status_code == 200
            raw_data = response.json()
            self._token_data = raw_data
            with open(self.token_filename, 'w') as token_handler:
                token_handler.write(json.dumps(raw_data))

    def prepare_headers(self):
        """Prepare headers for communication with the server"""

        return {'Authorization': 'Bearer {}'.format(
            self._token_data['access_token']),
                'Content-Type': 'application/json'}

    def get_from_server(self, url: str) -> dict:
        """Communicate with the server"""

        response = requests.get(url, headers=self.prepare_headers())
        if response.status_code == 401:
            self.refresh_token()
            response = requests.get(url, headers=self.prepare_headers())
            if response.status_code == 401:
                raise exceptions.TokenAndRefreshExpiredError
        assert response.status_code == 200
        return response.json()

    @property
    def badges(self) -> list:
        """Get list of badges from Server

        Example:

        >>> badgr = BadgrLite(token_filename='./token.json')
        >>> for badge in badgr.badges:
        ...     print(badge)
        ...
        cTjxL52HQBiSgIp5JuVq5w: Bay Area Python Interest Group TDD Participant
        5YhFytMUQb2loOMEy63gQA: Bay Area Python Interest Group TDD Quizmaster
        yzExTDvOTnOx_R3YhwPf3A: Test Driven Development Fundamentals Champion
        yNjcY70FSn603SO9vMGhBA: Install Python with Virtual Environments
        """
        self.load_token()
        raw_data = self.get_from_server(
            'https://api.badgr.io/v2/badgeclasses')['result']

        return [Badge(b) for b in raw_data]

    def _validate_award_badge_response(self, response: Response) -> None:
        """Review response from Badge().award and raise any exceptions"""
        # It's okay as a function here; pylint: disable=R0201

        if response.status_code == 404:
            raise exceptions.BadBadgeIdError(
                exceptions.BadBadgeIdError.__doc__)

        if response.status_code == 400:
            raise exceptions.AwardBadgeBadDataError(str(response.json()))

        assert response.status_code == 201 and\
            response.json()['status']['success']
        assert len(response.json()['result']) == 1

    def award_badge(self, badge_id: str, badge_data: dict) -> Badge:
        """Given a previously created badge_id and badge_data, award badge

        Example:

        >>> badgr = BadgrLite(token_filename='./token.json')
        >>> badge_data = {
        ...     "name": "Sample badge",
        ...     "recipient": {
        ...         "identity": "joe@example.com"
        ...     },
        ...     "notify": True,
        ...     "evidence": [{
        ...         "url": "http://example.com/",
        ...         "narrative": "Glen completed all the prereqs for..."
        ...     }]
        ... }
        >>>
        >>> badge_id = '2TfNNqMLT8CoAhfGKqSv6Q'
        >>> result = badgr.award_badge(badge_id, badge_data)
        >>> print(result)
        qv4DMvnYT0Gwz7wquRasvg: <No name>
        """

        self.load_token()
        base = 'https://api.badgr.io/v2'
        url = '{}/badgeclasses/{}/assertions'.format(base, badge_id)
        headers = self.prepare_headers()
        response = requests.post(url, headers=headers, json=badge_data)
        if response.status_code == 401:
            self.refresh_token()
            response = requests.post(url, headers=headers, json=badge_data)
            if response.status_code == 401:
                raise exceptions.TokenAndRefreshExpiredError

        self._validate_award_badge_response(response)
        return Badge(response.json()['result'][0])



# -*- coding: utf-8 -*-
# pylint: disable=E1120, R0913, R0903, C0103
"""Console script for badgr_lite."""


import click

from badgr_lite.models import BadgrLite
from badgr_lite import exceptions


class Config:
    """Share configuration accross commands"""

    def __init__(self):
        self.token_file = None


pass_config = click.make_pass_decorator(Config, ensure=True)


def xor(first: bool, second: bool) -> bool:
    """Return exclusive OR for boolean arguments"""

    return (first and not second) or (not first and second)


def ensure_evidence(badge_data: dict) -> dict:
    """Given badge_data, ensure 'evidence' key exists with list value"""

    if 'evidence' not in badge_data:
        badge_data['evidence'] = [{}]
    return badge_data


@click.group()
@click.option('--token-file', type=click.Path(),
              default='./token.json',
              help="File holding token credentials")
@pass_config
def main(config, token_file):
    """Automate Badgr tasks without the overhead of badgr-server"""

    config.token_file = token_file


@main.command()
@pass_config
def list_badges(config):
    """Pull and print a list of badges from server"""

    badgr = BadgrLite(token_filename=config.token_file)
    try:
        for badge in badgr.badges:
            click.echo(badge)
    except exceptions.TokenFileNotFoundError as err:
        for line in err.args:
            click.echo(line)


@main.command()
@pass_config
@click.option('--badge-id', prompt='Badge ID',
              help='ID of badge to award')
@click.option('--recipient', prompt='Recipient email',
              help='Email of recipient')
@click.option('--notify/--no-notify', default=False,
              help="Should badgr notify recipient?")
@click.option('--evidence-url',
              help="Optional evidence-url for awarded badge")
@click.option('--evidence-narrative',
              help="Optional evidence-narrative for awarded badge")
def award_badge(config, badge_id, recipient, notify,
                evidence_url, evidence_narrative):
    """Award badge with BADGE_ID to RECIPIENT.


    If evidence is provided, both --evidence-url and --evidence-narrative
    should be used.
    """

    if xor(evidence_url, evidence_narrative):
        raise click.UsageError(
            """If one evidence paramater is used, both are needed:
            --evidence-url and --evidence-narrative""")

    badge_data = {
        "recipient": {
            "identity": recipient,
        },
        "notify": notify,
    }

    if evidence_url:
        badge_data = ensure_evidence(badge_data)
        badge_data['evidence'][0]['url'] = evidence_url
        badge_data['evidence'][0]['narrative'] = evidence_narrative

    try:
        badgr = BadgrLite(token_filename=config.token_file)
        result = badgr.award_badge(badge_id, badge_data)
        click.echo(result)
    except exceptions.TokenFileNotFoundError as err:
        for line in err.args:
            click.echo(line)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Ignore methods that could be functions; pylint: disable=R0201

"""Tests for `badgr_lite` package."""


import datetime
import json
import os
from tempfile import mkdtemp
import unittest

import vcr

from badgr_lite.models import BadgrLite, Badge
from badgr_lite import exceptions


class BadgrLiteTestBase(unittest.TestCase):
    """BadgrLite setUp, tearDown and helper methods"""

    def setUp(self):
        """Create temporary input files of varying flavors"""

        self._tempdir = mkdtemp()
        self._sample_token = "FVQ__sample_token__QYzzRracgjH"
        self._sample_url = 'https://api.badgr.io/v2/badgeclasses'
        self.sample_token_file =\
            os.path.join(self._tempdir, "sample_token_file.json")

        with open(self.sample_token_file, 'w') as stf_h:
            stf_h.write(json.dumps(
                {"access_token": self._sample_token,
                 "token_type": "Bearer",
                 "refresh_token": "vK__sample_refresh_token__AlPZ"}
            ))

    def tearDown(self):
        """Remove temporary files"""

        os.remove(self.sample_token_file)
        os.rmdir(self._tempdir)

    def get_badgr_setup(self):
        """Return BadgrLite instance for testing"""

        badgr = BadgrLite(token_filename=self.sample_token_file)
        badgr.load_token()
        return badgr

    def get_sample_badge(self):
        """Fetch single badge for other tests"""

        badgr = self.get_badgr_setup()
        with vcr.use_cassette('tests/vcr_cassettes/badge_retrieval.yaml'):
            return badgr.badges[0]


class TestBadgeInstantiation(BadgrLiteTestBase):
    """Badge class instantiation tests"""

    def get_sample_attrs(self):
        """Return dictionary of test attributes for creating Badge"""

        return {
            'entity_type': 'BadgeClass',
            'entity_id': 'cTjxL52HQBiSgIp5JuVq5x',
            'open_badge_id': 'https://api.badgr.io/public/'
                             'badges/cTjxL52HQBiSgIp5JuVq5x',
            'created_at': '2019-09-04T19:03:24Z',
            'created_by': 'Lj__badge_creator__VIB',
            'description': 'sample badge description',
            'issuer': '5D__sample_issuer__4Kg',
            'issuer_open_badge_id': 'https://api.badgr.io/'
                                    'public/issuers/5D__sample_issuer__4Kg',
            'name': 'Sample badge for Unit Tests',
            'image': 'https://media.badgr.io/uploads/badges/'
                     'issuer_badgeclass_'
                     '488caae0-6fb7-42b5-b94e-d4ea0ac7d22d.png',
            'alignments': [],
            'expires': {'amount': None, 'duration': None},
            'criteria_narrative': 'Sample criteria narrative text',
            'criteria_url': 'http://example.com/',
            'tags': ['python', 'unit-test'],
            'extensions': {}
        }

    def test_instantiates_badge(self):
        """Badge() instantiates a Badge class"""

        attrs = self.get_sample_attrs()
        self.assertIsInstance(Badge(attrs), Badge)

    def test_has_required_attrs(self):
        """Badge has a list of required attributes for initialization"""

        for attr in ['entity_id', 'open_badge_id', 'created_at',
                     'created_by', 'issuer', 'issuer_open_badge_id',
                     'image', 'expires', 'extensions']:
            self.assertIn(attr, Badge.REQUIRED_ATTRS)

    def test_fails_if_required_attrs_not_included(self):
        """Badge() fails if required attributes not included"""

        with vcr.use_cassette('test/vcr_cassettes/badge_retrieval.yaml'):
            with self.assertRaises(exceptions.RequiredAttributesMissingError):
                # We need more attrs than just created_at
                Badge({'created_at': '2019-09-04T19:03:24Z'})


class TestBadgeBadgesMethod(BadgrLiteTestBase):
    """Badge.badges() related tests"""

    def test_should_give_a_list_for_badges(self):
        """Badge.badges() should give a list for badges"""

        badgr = self.get_badgr_setup()
        with vcr.use_cassette('tests/vcr_cassettes/badge_retrieval.yaml'):
            self.assertTrue(isinstance(badgr.badges, list))

    def test_should_contain_badge_classes(self):
        """Badge.badges() should contain badge classes"""

        badgr = self.get_badgr_setup()
        with vcr.use_cassette('tests/vcr_cassettes/badge_retrieval.yaml'):
            self.assertTrue(isinstance(badgr.badges[0], Badge))


class TestBadgeRequiredAttributes(BadgrLiteTestBase):
    """Badge() required attribute related tests"""

    def test_badge_should_have_entity_id(self):
        """Badge() should have an entity_id attribute"""

        badge = self.get_sample_badge()
        self.assertIsInstance(badge.entity_id, str)

    def test_badge_should_have_open_badge_id(self):
        """Badge() should have an open_badge_id attribute"""

        badge = self.get_sample_badge()
        self.assertIsInstance(badge.open_badge_id, str)

    def test_badge_should_have_created_at(self):
        """Badge() should have a created_at attribute"""

        badge = self.get_sample_badge()
        self.assertIsInstance(badge.created_at, datetime.datetime)

    def test_badge_should_have_created_by(self):
        """Badge() should have a created_by attribute"""

        badge = self.get_sample_badge()
        self.assertIsInstance(badge.created_by, str)

    def test_badge_should_have_issuer(self):
        """Badge() should have an issuer attribute"""

        badge = self.get_sample_badge()
        self.assertIsInstance(badge.issuer, str)

    def test_badge_should_have_issuer_open_badge_id(self):
        """Badge() should have an issuer_open_badge_id attribute"""

        badge = self.get_sample_badge()
        # It's a string, even though it often looks like a URL
        self.assertIsInstance(badge.issuer_open_badge_id, str)

    def test_badge_should_have_image(self):
        """Badge() should have an image attribute"""

        badge = self.get_sample_badge()
        # It's a string, even though it often looks like a URL
        self.assertIsInstance(badge.image, str)

    def test_badge_should_have_expires(self):
        """Badge() should have an expires attribute

        At the time of this writing, there isn't enough clarity about what the
        actual value of expires should be. The only assertion that can be made
        with confidence is that it exists.

        According to the OpenBadges FAQ (https://openbadges.org/faq/),

        "Some badges contain links to detailed evidence, expiration dates,
        searchable tags, and alignments to educational standards or
        frameworks."

        This would make one expect that it should be a datetime. However,
        instead of a datestring, the Badgr API returns:

        'expires': {'amount': None, 'duration': None},
        """

        badge = self.get_sample_badge()
        self.assertTrue(hasattr(badge, 'expires'))

    def test_badge_should_have_extensions(self):
        """Badge() should have an extensions attribute

        There are ways of extending OpenBadges:

        https://www.imsglobal.org/sites/default/
            files/Badges/OBv2p0Final/extensions/index.html

        However, we currently only ensure that the attribute exists.
        """

        badge = self.get_sample_badge()
        self.assertTrue(hasattr(badge, 'extensions'))


class TestBadgeOptionalAttributes(BadgrLiteTestBase):
    """Badge() optional() attribute related tests"""

    def test_badge_should_have_description(self):
        """Badge() may have an optional description attribute"""

        badge = self.get_sample_badge()
        self.assertIsInstance(badge.description, str)

    def test_badge_should_have_name(self):
        """Badge() may have an optional name attribute"""

        badge = self.get_sample_badge()
        self.assertIsInstance(badge.name, str)

    def test_badge_should_have_alignments(self):
        """Badge() may have an optional alignments attribute"""

        badge = self.get_sample_badge()
        self.assertIsInstance(badge.alignments, list)

    def test_badge_should_have_criteria_narrative(self):
        """Badge() may have an optoinal criteria_narrative attribute"""

        badge = self.get_sample_badge()
        self.assertIsInstance(badge.criteria_narrative, str)

    def test_badge_should_have_criteria_url(self):
        """Badge() may have an optional criteria_url attribute"""

        badge = self.get_sample_badge()
        # It's a string, even though it is used as a URL
        self.assertIsInstance(badge.criteria_url, str)

    def test_badge_should_have_tags(self):
        """Badge() may have an optional tags attribute"""

        badge = self.get_sample_badge()
        # It's a string, even though it is used as a URL
        self.assertIsInstance(badge.tags, list)


class TestBadgrLiteInstantiation(BadgrLiteTestBase):
    """Test BadgrLite Instantiation"""

    def test_instantiates_badgr_lite_class(self):
        """BadgrLite() instantiates a BadgrLite class"""
        badgr = self.get_badgr_setup()
        self.assertIsInstance(badgr, BadgrLite)

    def test_takes_a_token_file(self):
        """BadgrLite() takes a token file argument"""
        with self.assertRaises(TypeError):
            # TypeError: __init__() missing 1 required
            # positional argument: 'token_filename'

            # Intentionally no args; pylint: disable=E1120
            BadgrLite()

    def test_verifies_token_file_exists(self):
        """BadgrLite() verifies token file exists"""

        with self.assertRaises(exceptions.TokenFileNotFoundError):
            badgr = BadgrLite(token_filename='./non_existent_token_file.json')
            badgr.load_token()

    def test_verifies_token_file_contains_json(self):
        """BadgrLite() verifies token file exists"""

        with open(self.sample_token_file, 'w') as stf_h:
            stf_h.write("Bad JSON")

        with self.assertRaises(json.decoder.JSONDecodeError):
            badgr = BadgrLite(token_filename=self.sample_token_file)
            badgr.load_token()

    def test_verifies_bearer_token(self):
        """BadgrLite() has a bearer token when instantiated"""

        badgr = self.get_badgr_setup()

        # _token_data isn't meant to be exposed; pylint: disable=W0212
        self.assertEqual(badgr._token_data['token_type'], "Bearer")
        self.assertEqual(badgr._token_data['access_token'],
                         self._sample_token)

    @unittest.mock.patch('badgr_lite.models.BadgrLite.refresh_token')
    def test_attempts_to_refresh_token_when_appropriate(self, mock):
        """BadgrLite() attempts to refresh token when http 401"""

        badgr = self.get_badgr_setup()
        with vcr.use_cassette('tests/vcr_cassettes/try_refresh_token.yaml'):
            with self.assertRaises(exceptions.TokenAndRefreshExpiredError):
                badgr.get_from_server(self._sample_url)
        self.assertTrue(mock.called)

    def test_raises_token_expired_when_applicable(self):
        """BadgrLite() raises TokenExpired when applicable"""

        badgr = self.get_badgr_setup()
        with vcr.use_cassette('tests/vcr_cassettes/no_valid_auth_token.yaml'):
            with self.assertRaises(exceptions.TokenAndRefreshExpiredError):
                badgr.get_from_server(self._sample_url)

    def test_refreshes_token_when_expired(self):
        """BadgrLite() refreshes the token when it is expired"""

        badgr = self.get_badgr_setup()

        # _token_data isn't meant to be exposed; pylint: disable=W0212
        original_token = badgr._token_data['access_token']
        with vcr.use_cassette('tests/vcr_cassettes/expired_auth_token.yaml'):
            badgr.get_from_server(self._sample_url)
            self.assertNotEqual(original_token,
                                badgr._token_data['access_token'])


class TestBadgrLiteAwardMethod(BadgrLiteTestBase):
    """Test BadgrLite.award Method"""

    def get_sample_award_badge_data(self):
        """Sample data for .award() tests"""
        return {
            "recipient": {
                "identity": "joe@exmple.com"
            },
            "notify": True,
            "evidence": [{
                "url": "http://example.com/",
                "narrative": "Joe completed all..."
            }]
        }

    def get_sample_award_badge_id(self):
        """Sameple badge_id for .award() tests"""

        return '2TfNNqMLT8CoAhfGKqSv6Q'

    def test_award_badge_gives_badge_when_successful(self):
        """.award_badge() returns a badge when successful"""

        badgr = self.get_badgr_setup()

        with vcr.use_cassette('tests/vcr_cassettes/award_badge.yaml'):
            result = badgr.award_badge(
                self.get_sample_award_badge_id(),
                self.get_sample_award_badge_data())
            self.assertIsInstance(result, Badge)

    def test_award_badge_gives_error_when_given_bad_badge_id(self):
        """.award_badge() raises a BadBadgeIdError when given bad_id"""

        badgr = self.get_badgr_setup()

        with vcr.use_cassette('tests/vcr_cassettes/award_bad_badge_id.yaml'):
            with self.assertRaises(exceptions.BadBadgeIdError):
                badgr.award_badge('bad_badge_id',
                                  self.get_sample_award_badge_data())

    def test_award_badge_gives_error_when_given_bad_badge_data(self):
        """.award_badge() raises an wardBadgeBadDataError when given data

        Note that the details from the API are given to the exception for more
        details.
        """

        badgr = self.get_badgr_setup()

        with vcr.use_cassette('tests/vcr_cassettes/award_badge_bad_data.yaml'):
            with self.assertRaises(exceptions.AwardBadgeBadDataError):
                badgr.award_badge(
                    self.get_sample_award_badge_id(),
                    {'bad_badge_data': 1}
                )


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""BadgrLite Custom Exceptions"""


class TokenFileNotFoundError(BaseException):
    """Token file not found

    The token_filename argument that you passed into BadgrLite is not found.
    Please consider:

    - Using using `prime_initial_token` (see Installation instructions)
    - Checking the filename is correct
    - Ensuring the filename exists and is in JSON format
    """


class TokenAndRefreshExpiredError(BaseException):
    """Token and refresh expired

     The token has expired. We tried refreshing the token from the refresh
     token and are still not able to get authorization to work correctly.

     Use `prime_initial_token` (see Installation instructions) to reconfigure
     tokens.
     """


class RequiredAttributesMissingError(BaseException):
    """Required Badge Attributes Missing"""


class BadBadgeIdError(BaseException):
    """Award Badge given bad badge_id

    Please consider the badge ID that you are trying to award is correct.
    """


class AwardBadgeBadDataError(BaseException):
    """Award Badge given bad data"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['Click', 'requests', 'pytz']

setup_requirements = []

test_requirements = ['vcrpy', ]

setup(
    name='badgr_lite',
    version='1.0.1',
    author="Glen Jarvis",
    author_email='glen@glenjarvis.com',
    python_requires='!=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="Automate awarding Badgr (or equivalent) badges to recipients without the overhead of badgr-server",
    entry_points={
        'console_scripts': [
            'badgr=badgr_lite.cli:main',
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='OpenBadge',
    packages=find_packages(include=['badgr_lite', 'badgr_lite.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/glenjarvis/badgr_lite',
    zip_safe=False,
)



# -*- coding: utf-8 -*-

"""BadgrLite Helper functions"""

import re
import datetime

import pytz


UTC = pytz.timezone("UTC")
DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
DATETIME_MILLISECOND_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'


def pythonic(name: str) -> str:
    """Convert camelCase identifier to pythonic identifier

    Citaton: (https://stackoverflow.com/questions/1175208/
              elegant-python-function-to-convert-camelcase-
              to-snake-case/17328907)

    The Badgr API returns attributes in camel case (e.g., issuerOpenBadgeId).
    We wish to also see those attributes in a pythonic way
    (e.g., issuer_open_badgee_id).
    """
    regex_s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', regex_s1).lower()


def to_datetime(potential_datetime):
    """Given string, return UTC aware datetime"""

    final_datetime = potential_datetime
    if isinstance(potential_datetime, str):
        try:
            final_datetime = datetime.datetime.strptime(
                potential_datetime, DATETIME_FORMAT)
        except ValueError:
            final_datetime = datetime.datetime.strptime(
                potential_datetime, DATETIME_MILLISECOND_FORMAT)
        final_datetime = UTC.localize(final_datetime)
    return final_datetime
